#!/usr/bin/env python
"""
###############################################################################
# FILE NAME : flag_checker.py
# AUTHOR : J.Enochs
# CREATION DATE : 15-Sep-2016
# LAST MODIFIED : 29-Jul-2017
# DESCRIPTION : Accepts a 40 char sha1 hash (flag) and encodes it in a way that 
#               associates it with the user's MAC address. In other words, it
#               stamps the flag so the server knows who originally found it.
#               Why? - This enables the server to prevent students from sharing
#               their flags. If you submit someone else's flag, it's ignored.
###############################################################################/
"""

import os
import sys
import inspect
import optparse
import socket
import hashlib
import struct
import fcntl
import math
from socket import gethostbyname, gethostbyaddr, AF_INET, SOCK_STREAM
from uuid import getnode as get_mac

CODE4 = None  # last 4 digits of the user's MAC address

MAC = get_mac() # Don't switch NICs - IE. don't start using the CTF server using
                # a wireless connection then switch to a wired connection.
                
IPADDR = "10.50.21.227" # Set IP and port by placing a file called 'network' in
PORT = 5555     # same location as this script (line 1 = IP, line 2 = port). 
                # This hardcoded IP and port are only used if no 'network' file
                # is found.

def main(flags):
    """ main is called at the bottom of this script """    

    flags = str(flags)
    flags = "".join(flags)

    # If argv[1] is not present, return the student's score
    if flags is None:
        flags = "SCORE"
        
    # Determine if this flag has already been encoded.  If not, encode it now. 
    flag = validate(flags)

    # Submits the user-encoded flag to the server and returns the respone 
    flag_checker(flag)

def parse_options():
    """ Uses optparse to parse command line options """

    scan_range = False
    parser = optparse.OptionParser("usage%prog -H <target host> -p <target port>")
    parser.add_option("-f", dest='flags', default=None, type='string', help='specify a flag or submit multiple flags in a file named flags.')
    (options, args) = parser.parse_args()
    flags = options.flags
    return flags

def ip_of_iface(ifname):
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    return socket.inet_ntoa(fcntl.ioctl(
        s.fileno(),
        0x8915, # SIOCGIFADDR
        struct.pack('256s',ifname[:15])
        )[20:24]),socket.inet_ntoa(fcntl.ioctl(
        s.fileno(),
        0x891b,
        struct.pack('256s',ifname[:15])
        )[20:24])

def default_iface():
    route = '/proc/net/route'
    with open(route) as f:
        for line in f.readlines():
            try:
                iface,dest,_,flags,_,_,_,_,_,_,_, = line.strip().split()
                if dest != '00000000' or not int(flags, 16) & 2:
                    continue
                return iface
            except:
                continue

def write_network_config(ip,port):
    """ creates a file called network containing the server ip and port """

    filePath = pathfinder()

    with open(filePath,'w') as f:
        f.write(ip+'\n')
        f.write(port+'\n')

def find_server():
    """ scans the network looking for the flag server """

    global IPADDR, PORT
    ip,netmask = ip_of_iface(default_iface())
    network = struct.unpack('!L',socket.inet_aton(ip))[0] & struct.unpack('!L',socket.inet_aton(netmask))[0]
    subnet_range = struct.unpack('!L',socket.inet_aton('255.255.255.255'))[0] - struct.unpack('!L',socket.inet_aton(netmask))[0]

    print 'netmask:', netmask
    print 'network:', socket.inet_ntoa(struct.pack('!L',network))
    print 'subnet range:', subnet_range
    print 'hosts:', subnet_range
    
    success = False
    for h in range(1, subnet_range):
        addrToCheck = socket.inet_ntoa(struct.pack('!L',network+h))
        print addrToCheck
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            s.settimeout(1)
            s.connect((addrToCheck,5555))
            print 'connection successful at',addrToCheck
            success = True
            break
        except:
            continue

    if success == True:
        write_network_config(addrToCheck,'5555')
    else:
        write_network_config(IPADDR,PORT)

def pathfinder():
    """ determines the filepath to this script """

    global IPADDR, PORT
    fileName = "network"
    # determine if application is a script or frozen exe
    if getattr(sys, 'frozen', False):
        application_path = os.path.dirname(sys.executable)
    elif __file__:
        application_path = os.path.dirname(__file__)

    return os.path.join(application_path, fileName)

def validate(flags):
    """ determine if this flag has already been encoded """
    global IPADDR, PORT
    filePath = pathfinder()
    
    while not os.path.isfile(filePath):
        find_server()
    with open(filePath) as network_file:
        IPADDR = network_file.readline().strip()
        PORT = int(network_file.readline().strip())

    try:
        if len(flags) == 40:
            flag = make_unique_flag(flags)
        elif len(flags) == 44:
            flag = check_4digitID(flags)
        else:
            flag = flags
    except Exception as e:
        print e

    return flag

def flag_checker(flags):
    """ returns flag answer or hint """

    socket.setdefaulttimeout(2)
    s = socket.socket(AF_INET, SOCK_STREAM)
    try:
        s.connect((IPADDR, PORT))
        s.sendall(flags)
        while True:
            ans = s.recv(1024)
            if ans == '':
                break
            print ans, 
    except Exception as e:
        print e
        print "\nNo response from the flag server on IP {}.".format(IPADDR)
        return
    finally:
        s.close()
        del s

def check_4digitID(flag):
    """ check to ensure this user-encoded flag belongs to this user """

    global CODE4
    if CODE4 is None:
        mac_string = str(MAC)
        first_two_string = mac_string[-5:-3]
        last_two_string = mac_string[-2:]
        CODE4 = first_two_string + last_two_string
    front = flag[0:2]
    back = flag[-2:]
    code = front + back
    if code != CODE4:
        print "\nInvalid Code ID\n"
        exit(0)
    else:
        return flag

def make_unique_flag(flag):
    """ encodes the flag based on the user's mac address """

    global CODE4
    mac = MAC

    # hex to digits
    long_flag = int(flag, 16) #Hex string to long int

    # convert MAC to unique 4 digit int
    mac_string = str(mac)
    first_two_string = mac_string[-4:-2]
    last_two_string = mac_string[-2:]
    first_four_string = first_two_string + last_two_string
    CODE4 = first_four_string

    # XOR using 4 digit code generated above
    sha2 = hashlib.sha1()
    sha2.update(first_four_string) # need more than 4 digits to effectively XOR with a 40 
                       # char flag so we hash to get another 40 char flag
    anti_flag = sha2.hexdigest()
    anti_flag = int(anti_flag, 16) # convert new anti_flag from hex to digits
    modified_flag_decimal = long_flag ^ anti_flag # XOR both flags

    # convert our now modified flag back to hex
    hex_flag = hex(modified_flag_decimal)[2:-1]

    # Append 4 digit code to flag - server needs it to decode (undo our XOR)
    modified_flag_string = "{}{}{}".format(first_four_string[0:2],hex_flag,first_four_string[2:4])

    return modified_flag_string
    
if __name__ == "__main__":
    try:
        opt = sys.argv[1] 
    except:
        opt = "SCORE"
    main(opt)
