#!/usr/bin/env python
"""
###############################################################################
# FILE NAME : flag_checker.py
# AUTHOR : J.Enochs
# CREATION DATE : 15-Sep-2016
# LAST MODIFIED : 16-Feb-2018
# DESCRIPTION : Accepts a 40 char sha1 hash (flag) and encodes it in a way that 
#               associates it with the user's MAC address. In other words, it
#               stamps the flag so the server knows who originally found it.
#               Why? - This enables the server to prevent students from sharing
#               their flags. If you submit someone else's flag, it's ignored.
###############################################################################/
"""

import os
import sys
import inspect
import optparse
import socket
import hashlib
#import time
from socket import gethostbyname, gethostbyaddr, AF_INET, SOCK_STREAM
from uuid import getnode as get_mac
from netifaces import interfaces, ifaddresses, AF_INET

CODE4 = None  # last 4 digits of the user's MAC address

MAC = get_mac() # Don't switch NICs - IE. don't start using the CTF server using
                # a wireless connection then switch to a wired connection.
                
IPADDR = "192.168.242.215" # Set IP and port by placing a file called 'network' in
PORT = 5555     # same location as this script (line 1 = IP, line 2 = port). 
                # This hardcoded IP and port are only used if no 'network' file
                # is found.

def main(flags):
    """ control starts with the last 8 lines of this script """    

    flags = str(flags)
    flags = "".join(flags)

    # If argv[1] is not present, return the student's score
    if flags is None:
        flags = "SCORE"
        
    # Determine if this flag has already been encoded.  If not, encode it now. 
    flag = validate(flags)

    # Submits the user-encoded flag to the server and returns the respone 
    flag_checker(flag)

def parse_options():
    """Uses optparse to parse command line options"""

    scan_range = False
    parser = optparse.OptionParser("usage%prog -H <target host> -p <target port>")
    parser.add_option("-f", dest='flags', default=None, type='string', help='specify a flag or submit multiple flags in a file named flags.')
    (options, args) = parser.parse_args()
    flags = options.flags
    return flags

def validate(flags):
    """ Determine if this flag has already been encoded.  If not, encode it. """

    global IPADDR, PORT
    fileName = "network"
    # determine if application is a script or frozen exe
    if getattr(sys, 'frozen', False):
        application_path = os.path.dirname(sys.executable)
    elif __file__:
        application_path = os.path.dirname(__file__)

    filePath = os.path.join(application_path, fileName)
    
    if os.path.isfile(filePath):
        with open(filePath) as network_file:
            IPADDR = network_file.readline().strip()
            PORT = int(network_file.readline().strip())
    else:
        print "\nNo network file found! Using IP {} and port {}".format(IPADDR, PORT)

    try:
        if len(flags) == 40:
            flag = make_unique_flag(flags)
        elif len(flags) == 44:
            flag = check_4digitID(flags)
        else:
            flag = flags
    except Exception as e:
        print e

    return flag

def flag_checker(flags):
    """Returns flag answer or hint"""

    socket.setdefaulttimeout(2)
    s = socket.socket(AF_INET, SOCK_STREAM)
    try:
        s.connect((IPADDR, PORT))
        s.sendall(flags)
        while True:
            ans = s.recv(1024)
            if ans == '':
                break
            print ans, 
    except Exception as e:
        print e
        print "\nNo response from the flag server on IP {}.".format(IPADDR)
        return
    finally:
        s.close()
        del s

def check_4digitID(flag):
    """ Check to ensure this user-encoded flag belongs to this user """

    global CODE4
    if CODE4 is None:
        mac_string = str(MAC)
        first_two_string = mac_string[-4:-2]
        last_two_string = mac_string[-2:]
        CODE4 = first_two_string + last_two_string
    front = flag[0:2]
    back = flag[-2:]
    code = front + back
    if code != CODE4:
        print "\nInvalid Code ID\n"
        exit(0)
    else:
        return flag

def make_unique_flag(flag):
    global CODE4
    mac = MAC

    # hex to digits
    long_flag = int(flag, 16) #Hex string to long int

    # convert MAC to unique 4 digit int
    mac_string = str(mac)
    first_two_string = mac_string[-4:-2]
    last_two_string = mac_string[-2:]
    first_four_string = first_two_string + last_two_string
    CODE4 = first_four_string

    # XOR using 4 digit code generated above
    sha2 = hashlib.sha1()
    sha2.update(first_four_string) # need more than 4 digits to effectively XOR with a 40 
                       # char flag so we hash to get another 40 char flag
    anti_flag = sha2.hexdigest()
    anti_flag = int(anti_flag, 16) # convert new anti_flag from hex to digits
    modified_flag_decimal = long_flag ^ anti_flag # XOR both flags

    # convert our now modified flag back to hex
    hex_flag = hex(modified_flag_decimal)[2:-1]

    # Append 4 digit code to flag - server needs it to decode (undo our XOR)
    modified_flag_string = "{}{}{}".format(first_four_string[0:2],hex_flag,first_four_string[2:4])

    return modified_flag_string
    
if __name__ == "__main__":
    try:
        opt = sys.argv[1] 
    except:
        opt = "SCORE"
    main(opt)
