#!/usr/bin/env python
"""
###############################################################################
# FILE NAME : port_scanner.py
# AUTHOR : J.Enochs
# CREATION DATE : 15-Sep-2016
# LAST MODIFIED : 19-Oct-2016
# DESCRIPTION : Port scanner
#
###############################################################################/
"""

import os
import optparse
import socket
import time
from socket import gethostbyname, gethostbyaddr, AF_INET, SOCK_STREAM
from threading import Thread
from netifaces import interfaces, ifaddresses, AF_INET

BANNERS = []
PORTS = []
ADDRESSES = []

def main():
    target_host, port_list = parse_options()
    read_files()
    validate(target_host, port_list)
     
def parse_options():
    """Uses optparse to parse command line options"""
    scan_range = False
    parser = optparse.OptionParser("usage%prog -H <target host> -p <target port>")
    parser.add_option("-H", dest='target_host', default=None, type='string', help='specify target host or supply a file named addresses containing IPs')
    parser.add_option("-p", dest='port_list', default=None, type='string', help='specify target port/s. Place multiple ports in quotes & separated by commas. Optional if you have a file named ports in same location as this script.')
    (options, args) = parser.parse_args()
    target_host = options.target_host
    port_list = str(options.port_list).split(', ')
    return target_host, port_list

def get_banner(ip_addr, port):
    """Returns the application banner"""
    socket.setdefaulttimeout(3)
    s = socket.socket(AF_INET, SOCK_STREAM)
    try:
        s.connect((ip_addr, port))
        s.send('AdvPy\r\n')
        banner = s.recv(1024)
        print"{}/tcp open: {}\n".format(port, banner), 
        check_vuln_banners(banner)
    except:
        return
    finally:
        s.close()

def check_vuln_banners(banner):
    """Checks the banner against a list of vulnerable banners if available"""
    if not os.path.isfile('banners'):
        return
    if not os.access('banners', os.R_OK):
        print "ERROR: Access denied to banners file: Skipping banner check!"
    for line in BANNERS:
        if line.strip('\n') in banner:
            print " -- Vulnerabilty: {} --\n".format(line.strip('\n')),

def verify_subnet(target_host):
    """Checks Subnet

    Verifies the scanned IP is on the computer's subnet.  If not, it prints 
    a warning and skips that IP
    """
    base = target_host.rsplit('.', 1)[0]
    for ifaceName in interfaces():
        if ifaceName[0] == 'e':
            singleAddresses = [i['addr'] for i in ifaddresses(ifaceName).setdefault(AF_INET, [{'addr':'No IP addr'}] )]
            computer_base = singleAddresses[0].rsplit('.',1)[0]
            if base != computer_base:
                print "\n[+] Scan results for: {}".format(target_host)
                print " <<< You can't scan outside your subnet >>>"
                return False
            else:
                return True

def range_finder(target_host, port_list):
    """Iterates across the entire subnet (1 - 254) of IP addresses"""
    base = target_host.rsplit('.', 1)[0]
    for x in range(1,255):
        target_host = base + "." + str(x)
        port_scanner(target_host, port_list)

def read_files():
    """Reads local files into global variables

    This functions reads the banners, ports, and addresses files (if present)
    into global variables.
    """
    global BANNERS, PORTS, ADDRESSES
    if os.path.isfile('banners'):
        if not os.access('banners', os.R_OK):
            print "\nERROR: Access denied to banners file !!"
        else:
            f = open('banners', 'r')
            for line in f.readlines():
                BANNERS.append(line.strip('\n'))
            f.close
    if os.path.isfile('ports'):
        if not os.access('ports', os.R_OK):
            print "\nERROR: Access denied to ports file !!"
        else:
            f = open('ports', 'r')
            for line in f.readlines():
                PORTS.append(line.strip('\n'))
            f.close
    if os.path.isfile('addresses'):
        if not os.access('addresses', os.R_OK):
            print "\nERROR: Access denied to address file !!"
        else:
            f = open('addresses', 'r')
            for line in f.readlines():
                ADDRESSES.append(line.strip('\n'))
            f.close

def validate(target_host, port_list):
    """Validates the target_host and port_list

    First checks to see if these two required variables were provided
    on the command line.  If not, then it checks to see if they were
    provided via local files.  If either of these two variables were
    not supplied on the command line or in a file, the application 
    prints an error and exits.
    """
    is_good = True
    if port_list[0] == 'None':
        if not os.path.isfile('ports'):
            print "\nYou need to supply ports (-p) or provide a ports file\n"
            return
        else:
            port_list = []
            for line in PORTS:
                port_list.append(line.strip('\n'))
    if target_host is None:
        if not os.path.isfile('addresses'):
            print "\nYou need to supply a target host (-H) or an addresses file\n"
            return
        else:
            target_host = []
            for line in ADDRESSES:
                target_host.append(line.strip('\n'))
                is_good = True
                is_good = verify_subnet(line)
                if is_good:
                    port_scanner(line, port_list)
    else:
        port_scanner(target_host, port_list)
    
def port_scanner(target_host, port_list):
    """Scans the ports of the specified targets"""
    last_octet = target_host.rsplit('.', 1)[-1]
    if last_octet == '0':
        range_finder(target_host, port_list)
        return
    try:
        target_IP = gethostbyname(target_host)
    except:
        print "Cannot resolve {}: Unknown host".format(target_host)
        return
    try:
        target_name = gethostbyaddr(target_IP)
        print "\n[+] Scan results for : {} {}".format(target_IP, target_name[0])
    except:
        print "\n[+] Scan results for: {}".format(target_IP)
    for port in port_list:
        if len(port_list) > 100:
            time.sleep(0.0009)
        t = Thread(target = get_banner, args = (target_host, int(port)))
        t.start()

if __name__ == "__main__":
    main()
