#!/usr/bin/env python
"""
###############################################################################
# FILE NAME : flag_server.py
# AUTHOR : J.Enochs
# CREATION DATE : 07-Sep-2016
# LAST MODIFIED : 02-Feb-2017
# DESCRIPTION : Gives students points for scanning the active port and then 
#               changes the IP and port and waits for the next student. Gives
#               three points per flag with an adjustable max of 3 flags (12pts)
#
###############################################################################/
"""

import socket
import shelve
import re
import os
import random
from random import randint
from netifaces import interfaces, ifaddresses, AF_INET
from subprocess import call
from subprocess import Popen, PIPE
from uuid import getnode as get_mac

PORT_NUM = randint(2000,8000)
OCT4 = randint(202,250)
MACDICT = {}
MAC2SCORES = {}
FLAGCOUNTER = {}
CAPTUREDFLAGS = {}
FLAGLIMIT = 4
IPP = 0

def main():
    try:
        global PORT_NUM
        global MAC2SCORES
        global MACDICT
        global CAPTUREDFLAGS
        PORT_NUM = randint(2000,8000)
    except KeyboardInterrupt:
        sys.exit()
    try:
        db = shelve.open("../../Ex1_2_init_quiz_svr/initial_quiz_scores.db")
        if "macDict" in db:
            MACDICT = db["macDict"]
        else:
            print "\nMAC DATA FAILED TO LOAD"
            exit(0)
    except Exception as e:
        print e
        print "\nMAC DATA FAILED TO LOAD\n"
        exit(0)
    try:
        db2 = shelve.open("../class_flag_scores.db")
        if "mac2scores" not in db2:
            print "NO scores found!!"
            exit(0)
        else:
            MAC2SCORES = db2["mac2scores"]
    except Exception as e:
        print e
        print "Could not load scores!"
        exit(0)
    finally:
        db2.close()
    try:
        db3 = shelve.open("captured_flags.db")
        if "capturedFlags" in db3:
            CAPTUREDFLAGS = db3["capturedFlags"]
        else:
            print "\nCAPTURED FLAGS FAILED TO LOAD\n"
            exit(0)
    except Exception as e:
        print e
        print "\nCAPTURED FLAGS FAILED TO LOAD\n"
        exit(0)
    finally:
        db3.close()


    class Server():
        global OCT4
        global PORT_NUM
        def __init__(self,Address=('', PORT_NUM), MaxClient=5):
            self.s = socket.socket()
            self.s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            self.s.bind(Address)
            self.s.listen(MaxClient)

        def ScrambleIP(self):
            global IPP
            try:
                PORT_NUM = randint(2000,8000)
                OCT4 = randint(202,250)
                ip = "10.0.0." + str(OCT4)
                for ifaceName in interfaces():
                    if ifaceName[0] == 'w':
                        call(["sudo", "ifconfig", ifaceName, ip, "netmask", "255.255.255.0", "up"])
                        print "Flag server ex1 is running on {} at port {}".format(ip, PORT_NUM)
                        IPP = ip
            except Exception as e:
                print e
        
        def Get_MAC(self):
            global MAC2SCORES
            global FLAGCOUNTER
            global FLAGLIMIT
            if IPP not in self.Adr:
                try:
                    p = Popen(['ping', self.Adr[0], '-c1'], stdout=PIPE, stderr=PIPE)
                    out, err = p.communicate()
                    p = Popen(["arp", "-n", self.Adr[0]], stdout=PIPE)
                    s = p.communicate()[0]
                    mac = re.search(r"(([a-f\d]{1,2}\:){5}[a-f\d]{1,2})", s).groups()[0]
                except Exception as e:
                    print e
                    mac = "Err"
            else:
                mac = get_mac()
                
            if mac in MACDICT:
                name = MACDICT[mac]
            else:
                name = "Err"
            #key = name + "_PrtScn_" + PORT_NUM
            #print "{}".format(key)
            key = str(mac) + "PrtScn"
            if mac in MAC2SCORES:
                score = MAC2SCORES[mac]
            if key in CAPTUREDFLAGS:
                return name, mac, score 
            if mac in FLAGCOUNTER:
                count = FLAGCOUNTER[mac]
                if count < FLAGLIMIT:
                    count = count + 1
                    FLAGCOUNTER[mac] = count
                    if mac in MAC2SCORES:
                        score = MAC2SCORES[mac]
                        score = score + 3
                        MAC2SCORES[mac] = score
                    else:
                        score = 3
                        MAC2SCORES[mac] = score
                    #CAPTUREDFLAGS[key] = name
                else:
                    if name is not None:
                        CAPTUREDFLAGS[key] = name

                        if mac in MAC2SCORES:
                            score = MAC2SCORES[mac]
                        else:
                            print "MAX Flags but no score found! - inform instructor"
                            exit(0)
            else:
                FLAGCOUNTER[mac] = 1
                if mac in MAC2SCORES:
                    score = MAC2SCORES[mac]
                    score = score + 3
                    MAC2SCORES[mac] = score
                else:
                    score = 3
                    MAC2SCORES[mac] = score
                
            return name, mac, score
        def ProcessConnection(self):
            try:
                while True:
                    self.Client, self.Adr=(self.s.accept())
                    #print ('Got a connection from: '+str(self.Adr[0]))
                    name, mac, score = self.Get_MAC()
                    print "Name: {}".format(name)
                    print "MAC: {}".format(mac)
                    print "Score: {}".format(score)

                    if mac == "Err":
                        msg = " Error: Failed to detect MAC address - inform instructor"
                    elif name == "Err":
                        msg = " ERROR: Computer not listed in class database - inform instructor"
                    if mac == "Err" or name == "Err":
                        pass
                    else:
                        msg = "\n      <<<<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>>>\n              SUCCESS!! FLAG CAPTURED BY {}: {}        \n      <<<<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>>>\n".format(name.upper(), score)
                    self.Client.send(msg)
                    self.Client.close()
                    break
            except Exception as e:
                print e
            finally:
                self.s.close()
                db = shelve.open("../class_flag_scores.db")
                db["mac2scores"] = MAC2SCORES
                db.close()
                db2 = shelve.open("captured_flags.db")
                db2["capturedFlags"] = CAPTUREDFLAGS
                db2.close()
    try:
        while True:
            x = Server()
            x.ScrambleIP()
            x.ProcessConnection()
    except Exception as e:
        print e
    finally:
        for ifaceName in interfaces():
            if ifaceName[0] == 'w':
                call(["sudo", "ifconfig", ifaceName, "10.0.0.201", "netmask", "255.255.255.0", "up"])
                print "\n\nFlag server ex1 is running on 10.0.0.201"
                raw_input("Reset to original IP. Press any key to continue...")

    return


if __name__ == "__main__":
    main()
