#!/usr/bin/env python
"""
###############################################################################
# FILE NAME : update_server.py
# AUTHOR : J.Enochs
# CREATION DATE : 19-Sep-2016
# LAST MODIFIED : 29-Aug-2019
# DESCRIPTION : Flags for the C course
#
###############################################################################/
"""

import subprocess
import shelve
import operator
import hashlib
import os
import sys

FLAGS_DICT = {}
ROSTER = []
CAPTUREDFLAGS = {}
MULTI_FLAGS_DICT = {}
flag_key_list = []
key_display_index_dict = {}

def main():
    try:
        display_menu()
    except (KeyboardInterrupt, SystemExit):
        pass

def display_menu():
    global FLAGS_DICT
    global CAPTUREDFLAGS
    global key_display_index_dict
    global flag_key_list
    try:
        db = shelve.open("captured_flags.db")
      	if "capturedFlags" in db:
            CAPTUREDFLAGS = db["capturedFlags"]
     	else:
       	    print("CAPTURED FLAGS FAILED TO LOAD")
            exit(0)
            os.system("clear")
            print("\n\n   <<<<< CAPTURED FLAGS CLEARED >>>>>\n\n")
    except Exception as e:
        print(e)
        print("CAPTURED FLAGS FAILED TO LOAD")
        exit(0)
    finally:
        db.close()

    try:
        db = shelve.open("flags.db")
        FLAGS_DICT = db["flagsDict"]
    except:
        print("\nFAILED TO OPEN FLAG DB\n")
        exit(0)
    finally:
        db.close()
    flag_key_list = list(FLAGS_DICT)
    os.system("clear")
    while True:
        print("\nMake a selection:\n\n\)
            A - START / RESTART SERVER \n\
            B - STOP SERVER\n\n\
            C - DISPLAY CURRENT FLAGS\n\
            D - CLEAR CAPTURED FLAGS (!)\n\n\
            E - UPDATE SCORES\n\
            F - DELETE STUDENT (!)\n\n\
            S - SUM & DISPLAY SCORES\n\
            Z - CLEAR ALL SCORES AND FLAGS (!)\n\n\
            X - EXIT\n"
        ans = raw_input("Selection: ")
        ans = ans.lower()
    
        # RESTART SERVER
        if ans == 'a':
            os.system("clear")
            load_flag_base()
            restart_flag_server()
            save()
        # STOP SERVER
        elif ans == 'b':
            os.system("clear")
            stop_flag_server()
        # PRINT FLAG LIST
        elif ans == 'c':
            print_list(flag_key_list, key_display_index_dict) 
        # CLEAR CAPTURED FLAGS
        elif ans == 'd':
            clear_captured_flags()
        # UPDATE SCORES
        elif ans == 'e':
            update_scores()
        elif ans == 'f':
            delete_student()
        # SUM CAPTURED FLAGS
        elif ans == 's':
	    sum_captured_flags()
        # CLEAR QUIZ DATA
        elif ans == 'z':
            clear_quiz_data()
            load_flag_base()
            restart_flag_server()
        # EXIT
        elif ans == 'x':
            save()
            os.system("clear")
            break
        else:
            print("Invalid selection, try again.")
            raw_input("Press 'enter' to continue...")
            os.system("clear")

def restart_flag_server():
    stop_flag_server()
    os.system("nohup ./server.py >/dev/null 2>&1 </dev/null &")
    process = getpid()
    print("Restarted {}".format(process))

def getpid():
    os.system("pgrep -f 'python ./server.py' >/dev/null 2>&1")
    pid = os.popen("pgrep -f 'python ./server.py'").read().splitlines()
    pid.sort()
    if len(pid) > 0:
        return int(pid[0])
    else:
        return None

def stop_flag_server():
    killed = 0
    if getpid() != None:
        try:
            if killed == 0:
            	process = getpid()
            	os.kill(process, 0)
            	os.system("kill -9 " + str(process) + " >/dev/null 2>&1")
            	print("\nKilled {}".format(process))
            	killed = 1
        except:
            pass
    if killed == 0:
        print("\nServer not running")

def clear_captured_flags():
    qry = raw_input("You're about to delete all captured flags?  Press 'd' again to continue.  All other keys exit.\n")
    qry = qry.lower()
    if qry != 'd':
        exit(0)
    try:
        db = shelve.open("captured_flags.db")
        if "capturedFlags" in db:
            CAPTUREDFLAGS = db["capturedFlags"]
        else:
            print("CAPTURED FLAGS FAILED TO LOAD")
            exit(0)
        CAPTUREDFLAGS = {}
        db["capturedFlags"] = CAPTUREDFLAGS
        os.system("clear")
        print("\n\n   <<<<< CAPTURED FLAGS CLEARED >>>>>\n\n")
    except Exception as e:
        print(e)
        print("CAPTURED FLAGS FAILED TO LOAD")
        exit(0)
    finally:
        db.close()
                
def clear_quiz_data():
    qry = raw_input("You're about to delete all quiz data.  This information is used by several other applications.  Press 'z' again to continue.  All other keys exit.\n")
    qry = qry.lower()
    if qry != 'z':
        exit(0)
    try:
        db9 = shelve.open("../Quiz_Server/initial_quiz_scores.db")
        if "nameDict" in db9:
            db9["nameDict"] = {}
        if "macDict" in db9:
            db9["macDict"] = {}
        if "scoreDict" in db9:
            db9["scoreDict"] = {}
        if "scores" in db9:
            db9["scores"] = []
        if "registered" in db9:
            db9["registered"] = []
        os.system("clear")
        print("\n\n   <<<<< STUDENT RECORDS CLEARED >>>>>")
    except Exception as e:
        print(e)
        exit(0)
    finally:
        db9.close()

    try:
        db7 = shelve.open("../class_flag_scores.db")
        if "mac2scores" in db7:
            db7["mac2scores"] = {}
    except Exception as e:
        print(e)
        exit(0)
    finally:
        db9.close()
    try:
        db = shelve.open("captured_flags.db")
        if "capturedFlags" in db:
            CAPTUREDFLAGS = db["capturedFlags"]
        else:
            print("CAPTURED FLAGS FAILED TO LOAD")
            exit(0)
        CAPTUREDFLAGS = {}
        db["capturedFlags"] = CAPTUREDFLAGS
        print("   <<<<< CAPTURED FLAGS CLEARED >>>>>")
    except Exception as e:
        print(e)
        print("CAPTURED FLAGS FAILED TO LOAD")
        exit(0)
    finally:
        db.close()

def sum_captured_flags():
    global ROSTER
    load_flag_base()
    # Load class roster
    if os.path.isfile("roster.txt"):
        try:
            with open("roster.txt", "r") as roster_file:
                for line in roster_file:
                    ROSTER.append(line.lower().strip())
        except Exception as e:
            print(e)
            print("\nFailed to load class roster\n")
            exit(0)
    try:
        db8 = shelve.open("../class_flag_scores.db")
        if "mac2scores" in db8:
            class_scores = db8["mac2scores"]
        else:
            print("\nCOULD NOT LOAD SCORES\n")
            exit(0)
    except Exception as e:
        print(e)
        print("\nCOULD NOT LOAD SCORES\n")
        exit(0)
    finally:
        db8.close()
    try:
        db9 = shelve.open("../Quiz_Server/initial_quiz_scores.db")
        if "macDict" in db9:
            macDict = db9["macDict"]
        else:
            print("\nCOULD NOT LOAD MAC ADDRESSES\n")
            exit(0)
    except Exception as e:
        print(e)
        print("\nCOULD NOT LOAD MAC ADDRESSES\n")
        exit(0)
    finally:
        db9.close()

    for name in ROSTER:
        reportcard = {}
        score = 0
        flag = None
        exer = None
        pts = 0
	for key in CAPTUREDFLAGS:
            #print("\n{}\n".format(key))
	    if CAPTUREDFLAGS[key] == name:
	        if len(key) > 42:
     		    flag = unmake_unique_flag(key)
		    if flag in FLAGS_DICT:
		        exer = FLAGS_DICT[flag]
                        if exer in MULTI_FLAGS_DICT:
                            pts = MULTI_FLAGS_DICT[exer]
                        else:
                            pts = 100
                        if "NICE try" in exer:
                            continue
		    else:
		        exer = "Unknown: not found."
                        continue
                    mykey = name + "_" + exer + "  " + str(pts)
                    if exer not in reportcard.values():
                        if "-TryGit" not in exer:
                            if "-Active" not in exer:
                                reportcard[mykey] = exer
                                score = score + pts
		else:
		    exer = "Unknown: small"
                    reportcard[key] = exer
            elif name.lower() in key.lower():
                if "NICE try" in key:
                    continue
	        exer = CAPTUREDFLAGS[key]
                pts = 0
                if exer in MULTI_FLAGS_DICT:
                    pts = MULTI_FLAGS_DICT[exer]
                else:
                    pts = 100
                if exer not in reportcard.values():
                    if "-TryGit" not in exer:
                        if "-Active" not in exer:
                            myKey = key + "  " + str(pts)
                            reportcard[myKey] = exer
                            score = score + pts
	sorted_card = sorted(reportcard.items(), key=operator.itemgetter(1))
        for line in sorted_card:
            #print("{}_{}".format(name,reportcard[line])) 
            print("{}".format(line[0])) 
        print("{} score: {}\n".format(name,score))

def unmake_unique_flag(flag):

    # Count leading zeros

    # Remove 4 digit code
    code4 = flag[0:2] + flag[-2:]
    stripped_flag = flag[2:-2]

    # Modified flag from hex string back to long int
    long_flag = int(stripped_flag, 16)
    # Reverse XOR using magic 4
    sha2 = hashlib.sha1()
    sha2.update(code4)
    anti_flag = sha2.hexdigest()
    anti_flag = int(anti_flag, 16)
    flag_decimal = long_flag ^ anti_flag
    # Back to hex as the original flag
    original_flag = hex(flag_decimal)[2:-1]
    length = len(original_flag)
    if length < 40:
        num = 40 - length
        for _ in range(num):
            original_flag = "0" + original_flag

    return original_flag



def update_scores():
    try:
        db8 = shelve.open("../class_flag_scores.db")
        if "mac2scores" in db8:
            class_scores = db8["mac2scores"]
        else:
            print("\nCOULD NOT LOAD SCORES\n")
            exit(0)
    except Exception as e:
        print(e)
        print("\nCOULD NOT LOAD SCORES\n")
        exit(0)
    finally:
        db8.close()
    try:
        db9 = shelve.open("../Quiz_Server/initial_quiz_scores.db")
        if "macDict" in db9:
            macDict = db9["macDict"]
        else:
            print("\nCOULD NOT LOAD MAC ADDRESSES\n")
            exit(0)
    except Exception as e:
        print(e)
        print("\nCOULD NOT LOAD MAC ADDRESSES\n")
        exit(0)
    finally:
        db9.close()
    name2scores = {}
    for mac in class_scores:
        name2scores[macDict[mac]] = class_scores[mac]
    print("\n\n")
    count = 1
    validated_ans = ""
    as_displayed = []
    while True:
        os.system("clear")
        for x in name2scores:
            print("({}) - {}\nScore: {}\n\n".format(count, x, name2scores[x]))
            count = count + 1
            mac = (key for key,value in macDict.items() if value.lower() == x.lower()).next()
            tup = (x, name2scores[x], mac)
            as_displayed.append(tup)
        ans = raw_input("Make a selection (0 = exit): ")
        if ans.isdigit():
            ans = int(ans)
            if ans < count:
                validated_ans = ans
                break
            else:
                print("Failed: less than count check")
                raw_input("STOP")
        else:
            print("Failed is digit test")
            raw_input("STOP")
    if validated_ans == 0:
        os.system("clear")
        return
    else:
        while True:
            print("\nYou selected {} who has a current score of {}".format(as_displayed[ans -1][0], as_displayed[ans -1][1]))
            ans2 = raw_input("Enter new score or 'e' for exit: ")
            if ans2.isdigit():
                ans2 = int(ans2)
                class_scores[as_displayed[validated_ans - 1][2]] = ans2
                break
            elif ans2 == 'e':
                exit(0)
        try:
            db8 = shelve.open("../class_flag_scores.db")
            if "mac2scores" in db8:
                db8["mac2scores"] = class_scores
            else:
                print("\nCOULD NOT LOAD SCORES\n")
                exit(0)
        except Exception as e:
            print(e)
            print("\nCOULD NOT LOAD SCORES\n")
            exit(0)
        finally:
            db8.close()
    os.system("clear")
    return
            

def delete_student():
    try:
        db8 = shelve.open("../class_flag_scores.db")
        if "mac2scores" in db8:
            class_scores = db8["mac2scores"]
        else:
            print("\nCOULD NOT LOAD SCORES\n")
            exit(0)
    except Exception as e:
        print(e)
        print("\nCOULD NOT LOAD SCORES\n")
        exit(0)
    finally:
        db8.close()

    try:
        db9 = shelve.open("../Quiz_Server/initial_quiz_scores.db")
        if "macDict" in db9:
            mac_dict = db9["macDict"]
        else:
            print("\nCOULD NOT LOAD MAC ADDRESSES\n")
            exit(0)
        if "registered" in db9:
            registered_dict = db9["registered"]
        else:
            print("\nCOULD NOT LOAD REGISTERED\n")
            exit(0)
    except Exception as e:
        print(e)
        print("\nCOULD NOT LOAD MAC ADDRESSES\n")
        exit(0)
    finally:
        db9.close()

    try:
        db10 = shelve.open("captured_flags.db")
        if "capturedFlags" in db10:
            captured_dict = db10["capturedFlags"]
        else:
            print("\nCOULD NOT LOAD CAPTURED FLAGS\n")
            exit(0)
    except Exception as e:
        print(e)
        print("\nCOULD NOT LOAD CAPTURED FLAGS\n")
        exit(0)
    finally:
        db10.close()

    #mac_list = []
    #mac_list = list(macDict)
    #print(class_scores)
    #print(mac_dict)
    #print(registered_dict)
    #print(captured_dict)
    print("\n\n")
    #class_scores.pop('192.168.242.136', 0)
    #class_scores.pop('192.168.242.119', 0)
    captured_dict.pop('koehlmoos*', 0)
    captured_dict = {k: v for k, v in captured_dict.iteritems() if v != 'koehlmoos'}
    captured_dict = {k: v for k, v in captured_dict.iteritems() if v != 'rob'}
    #registered_dict.remove('koehlmoos')
    #registered_dict.remove('rob')
    #mac_dict.pop('192.168.242.136', 0)
    #mac_dict.pop('192.168.242.119', 0)


    #print(class_scores)
    print("\n")
    #print(mac_dict)
    #print(registered_dict)
    print(captured_dict)

    try:
        db8 = shelve.open("../class_flag_scores.db")
        if "mac2scores" in db8:
            db8["mac2scores"] = class_scores
        else:
            print("\nCOULD NOT UPDATE MAC1\n")
            exit(0)
    except Exception as e:
        print(e)
        print("\nCOULD NOT LOAD MAC1\n")
        exit(0)
    finally:
        db8.close()
    
    try:
        db11 = shelve.open("captured_flags.db")
        if "capturedFlags" in db11:
            db11["capturedFlags"] = captured_dict
        else:
            print("\nCOULD NOT UPDATE CAPTURED\n")
            exit(0)
    except Exception as e:
        print(e)
        print("\nCOULD NOT UPDATE CAPTURED\n")
        exit(0)
    finally:
        db11.close()

    try:
        db9 = shelve.open("../Quiz_Server/initial_quiz_scores.db")
        if "macDict" in db9:
            db9["macDict"] = mac_dict
        else:
            print("\nCOULD NOT UPDATE MAC2\n")
            exit(0)
        if "registered" in db9:
            db9["registered"] = registered_dict
        else:
            print("\nCOULD NOT UPDATE REGISTERED\n")
            exit(0)
    except Exception as e:
        print(e)
        print("\nCOULD NOT LOAD MAC2\n")
        exit(0)
    finally:
        db9.close()
    return

            
def load_flag_base():
    global FLAGS_DICT
    global MULTI_FLAGS_DICT
    """Reloads the class flags if shit gets fu$%ed up"""

    FLAGS_DICT = {} 
    MULTI_FLAGS_DICT = {}

    with open("flags.txt", "r") as flag_file:
        for line in flag_file:
            line = line.strip()
            if line:
                if not line.startswith("#"):
                    FLAGS_DICT[line.split("=")[1].strip()] = line.split("=")[2].strip()

    load_class_roster()

    #print(FLAGS_DICT)
    #exit(0)

    # ADVANCED 9pt FLAGS : 36 char length 
    #FLAGS_DICT["607e0412fec4942d5818d2104a0934406985"] = "Advanced Ex1"


    # REGULAR 3pt FLAGS : key = SHA1, value = exercise
    #FLAGS_DICT["fc078aef22e77264eb1448bbc2538d427dbb90d5"] = "Ex1_1 Local Repo"


    # NICE TRY 1pt FLAGS : value must contain "NICE"
    #FLAGS_DICT["1d774be4e8167ca3ece57c304cee31a2137df244"] = "NICE try! But no, that's not it.  That's the hash of the original file. If you 'echo' or 'cat' the data from one file to the other, the hash does not change.  I would like you to use the ascii command for writing data to a file. +1pt for trying the obvious!"
    

    # Hint Section : key must contain "Ex"
    #FLAGS_DICT["Ex1_5_h1"] = "Hint:  from astropy.io import ascii"
    FLAGS_DICT["Ex2_14"] = "Hint:  http://www.codeforwin.in/2015/07/c-program-to-find-sum-of-row-and-column-of-matrix.html"
    FLAGS_DICT["Ex2_15"] = "Hint:  http://www.w3resource.com/c-programming-exercises/pointer/c-pointer-exercise-8.php"
    FLAGS_DICT["Ex2_16"] = "Hint:  http://www.w3resource.com/c-programming-exercises/file-handling/c-file-handling-exercise-13.php"
    FLAGS_DICT["Ex2_17"] = "Hint:  http://www.w3resource.com/c-programming-exercises/function/c-function-exercise-8.php"
    FLAGS_DICT["Ex2_18"] = "Hint:  http://www.w3resource.com/c-programming-exercises/pointer/c-pointer-exercise-13.php"
    FLAGS_DICT["Ex2_19"] = "Hint:  http://www.w3resource.com/c-programming-exercises/for-loop/c-for-loop-exercises-57.php"
    FLAGS_DICT["Ex2_20"] = "Hint:  http://www.w3resource.com/c-programming-exercises/string/c-string-exercise-29.php"
    

    # INSTRUCTOR GRADED FLAGS : key = exercise, value = name list
    # FLAGS_DICT["Ex1_1 Active Contributor"] = ["elliott"]
    # Scavenger Hunt - Intermediate
    FLAGS_DICT["Ex1_1 Active"] = []
    FLAGS_DICT["Ex1_1-TryGit"] = []
    FLAGS_DICT["Ex1_2 Writing"] = []
    FLAGS_DICT["Ex2_2 Questions"] = []
    FLAGS_DICT["Ex2_3 Questions"] = []
    FLAGS_DICT["Ex2_4 Questions"] = []
    FLAGS_DICT["Ex2_5 Questions"] = []
    FLAGS_DICT["Ex2_6 Questions"] = []
    FLAGS_DICT["Ex2_7 Questions"] = []
    FLAGS_DICT["Ex2_8 Questions"] = []
    FLAGS_DICT["Ex2_9 Questions"] = []
    FLAGS_DICT["Ex2_10 Questions"] = []
    FLAGS_DICT["Ex2_11 Questions"] = []
    FLAGS_DICT["Ex2_12 Questions"] = []
    FLAGS_DICT["Ex2_13 Questions"] = []
    FLAGS_DICT["ExExtra Reverse It"] = ["maixner","shockley","Brogie"]
    FLAGS_DICT["ExExtra Reverse It 2"] = ["enochs"]

    
##############  Port flags  #######################
    MULTI_FLAGS_DICT["ExPort_8081"] = 0

##############  Maker flags  #######################
    # First maker flag = 6pts
    FLAGS_DICT["Class Maker"] = []
    # Maker flags 2-4 = 3pts for a total of 15 maker points
    FLAGS_DICT["Class Maker-2"] = []
    FLAGS_DICT["Class Maker-3"] = []
    FLAGS_DICT["Class Maker-4"] = []

##############  Special 3 point flags  #######################
    FLAGS_DICT["Ex2_19 Input Validation"] = []
    FLAGS_DICT["ExArd Unique Song"] = []
   
##############  Special 6 point flags  #######################
    FLAGS_DICT["Class Player"] = []
    FLAGS_DICT["Class Thinker"] = []
    FLAGS_DICT["Class Helper"] = []
    FLAGS_DICT["ExRPI Coder"] = []
    FLAGS_DICT["ExARD Interrupts"] = []
    FLAGS_DICT["ExARD Traffic Light Timing"] = []
    FLAGS_DICT["Ex2_21 Date Time"] = []
    FLAGS_DICT["Adv Assembly"] = []

##############  Special 9 point flags  #######################
    FLAGS_DICT["Q-Learning"] = []
    FLAGS_DICT["Ex2_16 Super Crypt"] = []
    FLAGS_DICT["Adv Morse"] = []
    FLAGS_DICT["Adv Trivia"] = []
    
##############  Adv Contributor 9 point flags #######################
    FLAGS_DICT["Adv Contributor"] = []
    FLAGS_DICT["Adv Contributor-1"] = []
    FLAGS_DICT["Adv Contributor-2"] = []
    FLAGS_DICT["Adv Contributor-3"] = []
    FLAGS_DICT["Adv Contributor-4"] = []
    FLAGS_DICT["Adv Contributor-5"] = []
    FLAGS_DICT["Adv Contributor-6"] = []

##############  Adjusted Point Section #######################
    MULTI_FLAGS_DICT["Ex2_2 Questions"] = 6
    MULTI_FLAGS_DICT["Ex2_3 Questions"] = 6
    MULTI_FLAGS_DICT["Ex2_5 Questions"] = 6
    MULTI_FLAGS_DICT["Ex2_6 Questions"] = 6
    MULTI_FLAGS_DICT["Ex2_11 Questions"] = 9
    MULTI_FLAGS_DICT["Ex2_13 Questions"] = 9

    MULTI_FLAGS_DICT["ExPort_1916"] = 9
    MULTI_FLAGS_DICT["ExPort_1970"] = 9
    MULTI_FLAGS_DICT["Ex2_14 Matrix Count"] = 6
    MULTI_FLAGS_DICT["Ex2_15 Permutations"] = 6
    MULTI_FLAGS_DICT["Ex2_16 Encrypt"] = 6
    MULTI_FLAGS_DICT["Ex2_16 Super Crypt"] = 9
    MULTI_FLAGS_DICT["Ex2_17 Largest Element"] = 6
    MULTI_FLAGS_DICT["Ex2_18 Buy a Vowel"] = 6
    MULTI_FLAGS_DICT["Ex2_19 Reverse"] = 6
    MULTI_FLAGS_DICT["Ex2_20 Brown Fox"] = 6
    MULTI_FLAGS_DICT["Ex2_21 Date Time"] = 6
    MULTI_FLAGS_DICT["Class Maker"] = 6
    MULTI_FLAGS_DICT["Class Player"] = 6
    MULTI_FLAGS_DICT["Class Thinker"] = 12
    MULTI_FLAGS_DICT["Class Helper"] = 6
    MULTI_FLAGS_DICT["ExRPI Coder"] = 6
    MULTI_FLAGS_DICT["ExARD Interrupts"] = 6
    MULTI_FLAGS_DICT["ExARD Traffic Light Timing"] = 6
    MULTI_FLAGS_DICT["Adv Morse"] = 9
    MULTI_FLAGS_DICT["Adv Morse Devens"] = 9
    MULTI_FLAGS_DICT["Adv Contributor"] = 9
    MULTI_FLAGS_DICT["Adv Assembly"] = 6
    MULTI_FLAGS_DICT["Adv Mem Location"] = 6
    MULTI_FLAGS_DICT["Q-Learning"] = 9
    MULTI_FLAGS_DICT["Adv Contributor-2"] = 9
    MULTI_FLAGS_DICT["Adv Contributor-3"] = 9
    MULTI_FLAGS_DICT["Adv Contributor-4"] = 9
    MULTI_FLAGS_DICT["Adv Contributor-5"] = 9
    MULTI_FLAGS_DICT["Adv Contributor-6"] = 9
    MULTI_FLAGS_DICT["Ex110 GCC"] = 100
    MULTI_FLAGS_DICT["Ex115 Makefiles"] = 100
    MULTI_FLAGS_DICT["Ex120 Fundamentals"] = 100
    MULTI_FLAGS_DICT["Ex130 Data Types"] = 100
    MULTI_FLAGS_DICT["Ex135 Constants"] = 100
    MULTI_FLAGS_DICT["Ex9135 Extra Credit"] = 100
    MULTI_FLAGS_DICT["Ex140 Control Flow"] = 100
    MULTI_FLAGS_DICT["Ex141 Algorithms"] = 300
    MULTI_FLAGS_DICT["Ex150 Arrays and Pointers"] = 100
    MULTI_FLAGS_DICT["Ex151 Arrays and Pointers"] = 100
    MULTI_FLAGS_DICT["Ex170 Strings"] = 100
    MULTI_FLAGS_DICT["Ex171 Strings"] = 100
    MULTI_FLAGS_DICT["Ex180 IO Streams and Files"] = 200
    MULTI_FLAGS_DICT["Ex182 IO User Input"] = 200
    MULTI_FLAGS_DICT["Ex184 IO Cmd Line"] = 200
    MULTI_FLAGS_DICT["Ex190 Functions"] = 100
    MULTI_FLAGS_DICT["Ex191 Functions"] = 100
    MULTI_FLAGS_DICT["Ex220 Memory"] = 300
    MULTI_FLAGS_DICT["Ex240 Data Structures"] = 200
    MULTI_FLAGS_DICT["Ex241 Data Structures"] = 300
    MULTI_FLAGS_DICT["ExExtra Reverse It"] = 300
    MULTI_FLAGS_DICT["ExExtra Reverse It 2"] = 300

    os.system("clear")
    print("\n\n      FLAGS RELOADED")
    return

def load_class_roster():
    """Loads the class roster into the flags database"""

    if os.path.isfile("roster.txt"):
        with open("roster.txt", "r") as roster_file:
            for line in roster_file:
                FLAGS_DICT[line.lower().strip()] = line.lower().strip()

def print_list(flag_key_list, key_display_index_dict):
    sf = sorted(FLAGS_DICT.items(), key=operator.itemgetter(1))
    os.system("clear")
    count = 0
    print("\n****** FLAGS ******\n")
    for key in flag_key_list:
        count = count + 1
        print("   {} - {}  {}".format(count, sf[count - 1][1], sf[count -1][0]))
        key_display_index_dict[count] = key
    print("\n*******************\n")
    raw_input("\n\nPress any key to continue...")
    os.system("clear")
    return count, key_display_index_dict

def save():
    try:
        db = shelve.open("flags.db")
        db["flagsDict"] = FLAGS_DICT
        db["multiDict"] = MULTI_FLAGS_DICT
    except:
        print("\nFAILED TO SAVE FLAG DB\n")
        exit(0)
    finally:
        db.close()

if __name__ == "__main__":
    main()
